class LatinCrillic {

  /// Method for converting from translit for the [source] value
  static String toCrillic(String source) {
    if (source.isEmpty) return source;
    var _complicatedSymbols = <String, String>{
      "ўғ": "o’g’",
      'ц': "ts",
      'Ш': 'Sh',
      'Ц': "S",
      'йў': "yo'",
      'йў': "yo`",
      'йў': "yo’",
      'Ё': 'Yo',
      'ё': "yo",
      'ю': "yu",
      'я': "ya",
      'ч': "ch",
      'Ч': "Ch",
      'Ш': "Sh",
      'Щ': "Sh",
      'ш': "sh",
      'Ю': "Yu",
      'Я': "Ya",
      'щ': "sh",
      'Ж': 'J',
      'Щ': 'Sh',
      'Ч': 'Ch',
      'Э': 'E',
      'Ю': 'Yu',
      'Я': 'Ya',
      'ё': 'yo',
      'ж': 'j',
      'щ': 'sh',
      'ш': 'sh',
      'ч': 'ch',
      "e": "e",
      "e": "e",
      'ъ': "'",
      'ю': 'yu',
      'я': 'ya',
      "ў": "o`",
      "ў": "o'",
      "ў": "o’",
      "Ў": "O’",
      "Ў": "O'",
      "Ў": "O`",
      "ғ": "g`",
      "Ғ": "G`",
      "ғ": "g’",
      "Ғ": "G’",

      "ғ": "g'",
      "Ғ": "G'",

    };
    var _transliteratedSymbol = <String, String>{
      'а': "a",
      'б': "b",
      'в': "v",
      'г': "g",
      'д': "d",
      'ж': "j",
      'з': "z",
      'ы': "i",
      'и': "i",
      'й': "y",
      'к': "k",
      'л': "l",
      'м': "m",
      'н': "n",
      'о': "o",
      'п': "p",
      'р': "r",
      'с': "s",
      'т': "t",
      'у': "u",
      'ф': "f",
      'х': "x",
      'ҳ': "h",
      "Ҳ": "H",
      'ъ': "'",
      "Х": "X",
      "қ": "q",
      "Қ": "Q",
      'А': "A",
      'Б': "B",
      'В': "V",
      'Г': "G",
      'Д': "D",
      'Е': "E",
      'Ж': "J",
      'З': "Z",
      'Й': "J",
      'К': "K",
      'Л': "L",
      'М': "M",
      'Н': "N",
      'О': "O",
      'П': "P",
      'Р': "R",
      'С': "S",
      'Т': "T",
      'У': "U",
      'Ф': "F",
      'Х': "X",
      'Ъ': "J",
      'Ы': "I",
      'И': "I",
      'Ь': "J",
      'Э': "E",
      " ": " ",
    };

    var regExp = RegExp(
      r'([a-z]+)',
      caseSensitive: false,
      multiLine: true,
    );

    if (!regExp.hasMatch(source)) return source;

    final sourceSymbols = <String>[];
    final toCrillic = <String>[];
    var deTransliteratedSymbol = <String, String>{};

    _complicatedSymbols.forEach((key, value) {
      source = source.replaceAll(value, key);
    });

    sourceSymbols.addAll(source.split(''));

    _transliteratedSymbol.forEach((key, value) {
      deTransliteratedSymbol[value] = key;
    });

    for (final element in sourceSymbols) {
      final transElement = deTransliteratedSymbol.containsKey(element)
          ? deTransliteratedSymbol[element] ?? ''
          : element;
      toCrillic.add(transElement);
    }

    return toCrillic.join();
  }

  /// Method for converting to translit for the [source] value
  static String toLatin(String source) {
    if (source.isEmpty) return source;
    var _transliteratedSymbol = <String, String>{
      'а': "a",
      'б': "b",
      'в': "v",
      'г': "g",
      'д': "d",
      'ж': "j",
      'з': "z",
      'ы': "i",
      'и': "i",
      'й': "y",
      'к': "k",
      'л': "l",
      'м': "m",
      'н': "n",
      'о': "o",
      'п': "p",
      'р': "r",
      'с': "s",
      'т': "t",
      'у': "u",
      'ф': "f",
      'х': "x",
      'ҳ': "h",
      "Ҳ": "H",
      'ъ': "'",
      "Х": "X",
      "қ": "q",
      "Қ": "Q",
      'А': "A",
      'Б': "B",
      'В': "V",
      'Г': "G",
      'Д': "D",
      'Е': "E",
      'Ж': "J",
      'З': "Z",
      'Й': "J",
      'К': "K",
      'Л': "L",
      'М': "M",
      'Н': "N",
      'О': "O",
      'П': "P",
      'Р': "R",
      'С': "S",
      'Т': "T",
      'У': "U",
      'Ф': "F",
      'Х': "X",
      'Ъ': "J",
      'Ы': "I",
      'И': "I",
      'Ь': "J",
      'Э': "E",
      " ": " ",
    };
    var _complicatedSymbols = <String, String>{
      'ц': "ts",
      'Ш': 'Sh',
      'Ц': "S",
      'йў': "yo'",
      'йў': "yo`",
      'йў': "yo’",
      'Ё': 'Yo',
      'ё': "yo",
      'ю': "yu",
      'я': "ya",
      'ч': "ch",
      'Ч': "Ch",
      'Ш': "Sh",
      'Щ': "Sh",
      'ш': "sh",
      'Ю': "Yu",
      'Я': "Ya",
      'щ': "sh",
      'Ж': 'J',
      'Щ': 'Sh',
      'Ч': 'Ch',
      'Э': 'E',
      'Ю': 'Yu',
      'Я': 'Ya',
      'ё': 'yo',
      'ж': 'j',
      'щ': 'sh',
      'ш': 'sh',
      'ч': 'ch',
      "э": "e",
      "e": "e",
      'ъ': "'",
      'ю': 'yu',
      'я': 'ya',
      "ў": "o`",
      "ў": "o'",
      "ў": "o’",
      "Ў": "O’",
      "Ў": "O'",
      "Ў": "O`",
      "ғ": "g`",
      "Ғ": "G`",
      "ғ": "g’",
      "Ғ": "G’",
      "ғ": "g'",
      "Ғ": "G'",

    };
    final regExp = RegExp(
      r'([а-я]+)',
      caseSensitive: false,
      multiLine: true,
    );

    if (!regExp.hasMatch(source)) return source;

    final translit = <String>[];
    final sourceSymbols = <String>[...source.split('')];

    _transliteratedSymbol.addAll(_complicatedSymbols);

    for (final element in sourceSymbols) {
      final transElement = _transliteratedSymbol.containsKey(element)
          ? _transliteratedSymbol[element] ?? ''
          : element;
      translit.add(transElement);
    }

    return translit.join();
  }
}
//import 'package:latin_crillic/src/latinandcrillic.dart';